"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BsyErrorHandlerFilter = void 0;
const common_1 = require("@nestjs/common");
const bsy_error_1 = require("bsy-error");
let BsyErrorHandlerFilter = class BsyErrorHandlerFilter {
    catch(exception, host) {
        const err = this.normalizeError(exception);
        const ctx = host.switchToHttp();
        const response = ctx.getResponse();
        switch (err.code) {
            case 'VALIDATION_ERROR':
            case 'VAL_ERROR_LIST':
                response
                    .status(common_1.HttpStatus.BAD_REQUEST)
                    .json(err);
                break;
            case 'UNAUTHORIZED_ERROR':
                response
                    .status(common_1.HttpStatus.UNAUTHORIZED)
                    .json(err);
                break;
            case 'NOT_FOUND_ERROR':
                response
                    .status(common_1.HttpStatus.NOT_FOUND)
                    .json(err);
                break;
            case 'FORBIDDEN_ERROR':
                response
                    .status(common_1.HttpStatus.FORBIDDEN)
                    .json(err);
                break;
            case 'DUPE_ERROR':
                response
                    .status(common_1.HttpStatus.CONFLICT)
                    .json(err);
                break;
            case 'PAYMENT_REQUIRED_ERROR':
                response
                    .status(common_1.HttpStatus.PAYMENT_REQUIRED)
                    .json(err);
                break;
            default:
                this.onUnhandledError(err, host);
                break;
        }
    }
    onUnhandledError(err, host) {
        const ctx = host.switchToHttp();
        const response = ctx.getResponse();
        console.error('Unhandled error occurred.');
        console.error(err);
        response
            .status(common_1.HttpStatus.INTERNAL_SERVER_ERROR)
            .json({
            detail: 'Oops, an error occurred.  The software development team has been notified.',
            code: 'UNHANDLED_ERROR'
        });
    }
    normalizeError(err) {
        if (err instanceof bsy_error_1.DetailedError)
            return err;
        if (err instanceof common_1.HttpException) {
            const httpErr = err;
            let detailedError;
            switch (err.getStatus()) {
                case common_1.HttpStatus.BAD_REQUEST:
                    detailedError = new bsy_error_1.ValidationError(httpErr.message, 'request');
                    break;
                case common_1.HttpStatus.NOT_FOUND:
                    detailedError = new bsy_error_1.NotFoundError(httpErr.message);
                    break;
                case common_1.HttpStatus.FORBIDDEN:
                    detailedError = new bsy_error_1.ForbiddenError(httpErr.message);
                    break;
                case common_1.HttpStatus.UNAUTHORIZED:
                    detailedError = new bsy_error_1.UnauthorizedError(httpErr.message);
                    break;
                default:
                    detailedError = new bsy_error_1.DetailedError(httpErr.message, 'UNKNOWN_HTTP_ERROR');
                    break;
            }
            detailedError.stack = err.stack;
            return detailedError;
        }
        if (err.message) {
            const detailedError = new bsy_error_1.DetailedError(err.message, 'UNHANDLED_ERROR');
            detailedError.stack = err.stack;
            return detailedError;
        }
        console.error('Failed to normalize error');
        console.error(err);
        return new bsy_error_1.DetailedError('Unknown error.', 'UNKNOWN_ERROR');
    }
};
BsyErrorHandlerFilter = __decorate([
    common_1.Catch()
], BsyErrorHandlerFilter);
exports.BsyErrorHandlerFilter = BsyErrorHandlerFilter;
//# sourceMappingURL=bsy-error-handler.filter.js.map