"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Dao = void 0;
const formn_1 = require("formn");
const bsy_error_1 = require("bsy-error");
class Dao {
    constructor(dataContext, Entity, alias) {
        this.dataContext = dataContext;
        this.Entity = Entity;
        this.alias = alias;
    }
    create(model) {
        return __awaiter(this, void 0, void 0, function* () {
            yield new formn_1.InsertModelValidator()
                .validate(model, this.Entity);
            try {
                return yield this.dataContext
                    .insert(this.Entity, model)
                    .execute();
            }
            catch (err) {
                if (err.message.startsWith('Cannot add or update a child row: a foreign key constraint fails')) {
                    const table = err.message.replace(/^.*REFERENCES `(\w+)`.*$/, '$1');
                    throw new bsy_error_1.NotFoundError(`Failed to create "${this.Entity.name}."  Invalid reference to "${table}."`);
                }
                throw err;
            }
        });
    }
    retrieve(cond, params) {
        if (cond instanceof formn_1.ParameterizedCondition) {
            params = cond.getParams();
            cond = cond.getCond();
        }
        const query = this.dataContext
            .from(this.Entity, this.alias);
        if (cond && Object.keys(cond).length)
            query.where(cond, params);
        const select = query.select();
        return select.execute();
    }
    retrieveById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const pk = formn_1.metaFactory
                .getColumnStore()
                .getPrimaryKey(this.Entity);
            if (pk.length !== 1)
                throw new Error('Dao.retrieveById does not support composite primary keys.');
            yield new formn_1.DeleteModelValidator()
                .validate({ [pk[0].mapTo]: id }, this.Entity);
            const cond = new formn_1.ConditionBuilder()
                .eq(`${this.alias}.${pk[0].mapTo}`, `:${pk[0].mapTo}`, id);
            const resources = yield this.retrieve(cond);
            if (resources.length === 0) {
                throw new bsy_error_1.NotFoundError(`"${pk[0].Entity.name}" not found using id "${id}."`);
            }
            else if (resources.length === 1)
                return resources[0];
            else {
                throw new bsy_error_1.DuplicateError(`Multiple "${pk[0].Entity.name}" resources found matching id "${id}."`, 'id', id);
            }
        });
    }
    updateModel(model) {
        return __awaiter(this, void 0, void 0, function* () {
            yield new formn_1.UpdateModelValidator()
                .validate(model, this.Entity);
            try {
                return yield this.dataContext
                    .update(this.Entity, model)
                    .execute();
            }
            catch (err) {
                if (err.message && err.message === 'Update operation did not affect any rows.') {
                    throw new bsy_error_1.NotFoundError(`"${this.Entity.name}" not found.`);
                }
                else if (err.message.startsWith('Cannot add or update a child row: a foreign key constraint fails')) {
                    const table = err.message.replace(/^.*REFERENCES `(\w+)`.*$/, '$1');
                    throw new bsy_error_1.NotFoundError(`Failed to update "${this.Entity.name}."  Invalid reference to "${table}."`);
                }
                throw err;
            }
        });
    }
    deleteById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const pk = formn_1.metaFactory
                .getColumnStore()
                .getPrimaryKey(this.Entity);
            if (pk.length !== 1)
                throw new Error('Dao.deleteById does not support composite primary keys.');
            const model = { [pk[0].mapTo]: id };
            yield new formn_1.DeleteModelValidator()
                .validate(model, this.Entity);
            try {
                return yield this.dataContext
                    .delete(this.Entity, model)
                    .execute();
            }
            catch (err) {
                if (err.message && err.message === 'Delete operation did not affect any rows.')
                    throw new bsy_error_1.NotFoundError(`"${this.Entity.name}" not found.`);
                throw err;
            }
        });
    }
    delete(cond, params) {
        if (cond instanceof formn_1.ParameterizedCondition) {
            params = cond.getParams();
            cond = cond.getCond();
        }
        const query = this.dataContext
            .from(this.Entity, this.alias);
        if (cond && Object.keys(cond).length)
            query.where(cond, params);
        const del = query.delete();
        return del.execute();
    }
}
exports.Dao = Dao;
//# sourceMappingURL=dao.js.map