"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SearchDao = void 0;
const formn_1 = require("formn");
const search_result_1 = require("./search-result");
class SearchDao {
    constructor(dataContext, Entity, alias) {
        this.dataContext = dataContext;
        this.Entity = Entity;
        this.alias = alias;
    }
    getSearchQuery() {
        return this.dataContext
            .from(this.Entity, this.alias);
    }
    getUniqueIdentifier() {
        const pks = formn_1.metaFactory
            .getColumnStore()
            .getPrimaryKey(this.Entity);
        if (pks.length !== 1) {
            throw new Error(`A unique identifier cannot be derived for "${this.Entity.name}" ` +
                'because it uses a composite primary key.');
        }
        return pks[0].mapTo;
    }
    retrieve(offset, rowCount, cond, order) {
        if (!order || !order.length) {
            const pks = formn_1.metaFactory
                .getColumnStore()
                .getPrimaryKey(this.Entity);
            order = pks
                .map(pk => {
                return {
                    property: formn_1.ColumnMetadata.createFQName(this.alias, pk.mapTo),
                    dir: 'ASC'
                };
            });
        }
        return this.dataContext.beginTransaction(() => __awaiter(this, void 0, void 0, function* () {
            const uniqueId = this.getUniqueIdentifier();
            const fqUniqueId = formn_1.ColumnMetadata.createFQName(this.alias, uniqueId);
            const idQuery = this.getSearchQuery();
            if (cond)
                idQuery.where(cond);
            const distinctEnts = yield idQuery
                .select(fqUniqueId)
                .orderBy(...order)
                .execute();
            const result = new search_result_1.SearchResult();
            result.count = distinctEnts.length;
            result.offset = offset;
            result.order = order;
            if (result.count && result.count > offset) {
                const ids = distinctEnts
                    .slice(offset, offset + rowCount)
                    .map(ent => ent[uniqueId]);
                const cb = new formn_1.ConditionBuilder();
                const idCond = cb
                    .in(fqUniqueId, ':search-distinct-ids', ids);
                const entities = yield this
                    .getSearchQuery()
                    .where(idCond)
                    .select()
                    .orderBy(...order)
                    .execute();
                result.rowCount = entities.length;
                result.entities = entities;
            }
            else {
                result.rowCount = 0;
                result.entities = [];
            }
            return result;
        }));
    }
}
exports.SearchDao = SearchDao;
//# sourceMappingURL=search-dao.js.map