"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SearchService = void 0;
const formn_1 = require("formn");
const bsy_error_1 = require("bsy-error");
class SearchService {
    constructor(dao, columnLookup) {
        this.dao = dao;
        this.columnLookup = columnLookup;
    }
    retrieve(offset = 0, rowCount = 10, cond, params, order) {
        cond = this.mapCondition(cond, params);
        order = this.mapOrder(order);
        return this.dao
            .retrieve(offset, rowCount, cond, order);
    }
    mapCondition(cond, params = {}) {
        if (!cond || !Object.keys(cond).length)
            return;
        try {
            cond = new formn_1.ConditionMapper()
                .map(cond, this.columnLookup, params);
            return formn_1.ParameterizedCondition.normalize(cond, params);
        }
        catch (err) {
            throw new bsy_error_1.ValidationError(err.message, 'cond');
        }
    }
    mapOrder(orders = []) {
        try {
            return orders
                .map(order => {
                return {
                    property: this.columnLookup
                        .getColumn(order.property),
                    dir: order.dir,
                };
            });
        }
        catch (err) {
            throw new bsy_error_1.ValidationError(err.message, 'order');
        }
    }
}
exports.SearchService = SearchService;
//# sourceMappingURL=search-service.js.map